@@ -85,6 +85,42 @@ get_provider_features (GoaProvider *provider)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static char *
+uri_to_string_with_path (SoupURI *soup_uri, const gchar *path)
+{
+  gchar *uri_string;
+  gchar *uri_tmp;
+
+  if (soup_uri == NULL)
+    return NULL;
+
+  uri_tmp = soup_uri_to_string (soup_uri, FALSE);
+  uri_string = g_strconcat (uri_tmp, path, NULL);
+  g_free (uri_tmp);
+
+  return uri_string;
+}
+
+static char *get_webdav_uri (SoupURI *soup_uri)
+{
+  gchar *uri_webdav;
+  const gchar *scheme;
+
+  if (soup_uri == NULL)
+    return NULL;
+
+  scheme = soup_uri_get_scheme (soup_uri);
+  if (g_strcmp0 (scheme, SOUP_URI_SCHEME_HTTPS) == 0)
+    soup_uri_set_scheme (soup_uri, "davs");
+  else
+    soup_uri_set_scheme (soup_uri, "dav");
+  uri_webdav = uri_to_string_with_path (soup_uri, WEBDAV_ENDPOINT);
+  /* restore initial scheme */
+  soup_uri_set_scheme (soup_uri, scheme);
+
+  return uri_webdav;
+}
+
 static gboolean on_handle_get_password (GoaPasswordBased      *interface,
                                         GDBusMethodInvocation *invocation,
                                         const gchar           *id,
@@ -168,17 +204,7 @@ build_object (GoaProvider         *provider,
         {
           gchar *uri_caldav;
 
-          uri_caldav = NULL;
-
-          if (uri != NULL)
-            {
-              gchar *uri_tmp;
-
-              uri_tmp = soup_uri_to_string (uri, FALSE);
-              uri_caldav = g_strconcat (uri_tmp, CALDAV_ENDPOINT, NULL);
-              g_free (uri_tmp);
-            }
-
+          uri_caldav = uri_to_string_with_path (uri, CALDAV_ENDPOINT);
           calendar = goa_calendar_skeleton_new ();
           g_object_set (G_OBJECT (calendar),
                         "accept-ssl-errors", accept_ssl_errors,
@@ -203,17 +229,7 @@ build_object (GoaProvider         *provider,
         {
           gchar *uri_carddav;
 
-          uri_carddav = NULL;
-
-          if (uri != NULL)
-            {
-              gchar *uri_tmp;
-
-              uri_tmp = soup_uri_to_string (uri, FALSE);
-              uri_carddav = g_strconcat (uri_tmp, CARDDAV_ENDPOINT, NULL);
-              g_free (uri_tmp);
-            }
-
+          uri_carddav = uri_to_string_with_path (uri, CARDDAV_ENDPOINT);
           contacts = goa_contacts_skeleton_new ();
           g_object_set (G_OBJECT (contacts),
                         "accept-ssl-errors", accept_ssl_errors,
@@ -256,24 +272,7 @@ build_object (GoaProvider         *provider,
         {
           gchar *uri_webdav;
 
-          uri_webdav = NULL;
-
-          if (uri != NULL)
-            {
-              const gchar *scheme;
-              gchar *uri_tmp;
-
-              scheme = soup_uri_get_scheme (uri);
-              if (g_strcmp0 (scheme, SOUP_URI_SCHEME_HTTPS) == 0)
-                soup_uri_set_scheme (uri, "davs");
-              else
-                soup_uri_set_scheme (uri, "dav");
-
-              uri_tmp = soup_uri_to_string (uri, FALSE);
-              uri_webdav = g_strconcat (uri_tmp, WEBDAV_ENDPOINT, NULL);
-              g_free (uri_tmp);
-            }
-
+          uri_webdav = get_webdav_uri (uri);
           files = goa_files_skeleton_new ();
           g_object_set (G_OBJECT (files),
                         "accept-ssl-errors", accept_ssl_errors,
@@ -319,6 +318,7 @@ build_object (GoaProvider         *provider,
  out:
   g_clear_object (&calendar);
   g_clear_object (&contacts);
+  g_clear_object (&documents);
   g_clear_object (&files);
   g_clear_object (&password_based);
   g_clear_pointer (&uri, (GDestroyNotify *) soup_uri_free);
@@ -1062,45 +1062,6 @@ refresh_account (GoaProvider    *provider,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-static void
-show_account (GoaProvider         *provider,
-              GoaClient           *client,
-              GoaObject           *object,
-              GtkBox              *vbox,
-              GtkGrid             *grid,
-              G_GNUC_UNUSED GtkGrid *dummy)
-{
-  gint row;
-
-  row = 0;
-
-  goa_util_add_account_info (grid, row++, object);
-
-  goa_util_add_row_switch_from_keyfile_with_blurb (grid, row++, object,
-                                                   /* Translators: This is a label for a series of
-                                                    * options switches. For example: “Use for Mail”. */
-                                                   _("Use for"),
-                                                   "calendar-disabled",
-                                                   _("Cale_ndar"));
-
-  goa_util_add_row_switch_from_keyfile_with_blurb (grid, row++, object,
-                                                   NULL,
-                                                   "contacts-disabled",
-                                                   _("_Contacts"));
-
-  goa_util_add_row_switch_from_keyfile_with_blurb (grid, row++, object,
-                                                   NULL,
-                                                   "documents-disabled",
-                                                   _("_Documents"));
-
-  goa_util_add_row_switch_from_keyfile_with_blurb (grid, row++, object,
-                                                   NULL,
-                                                   "files-disabled",
-                                                   _("_Files"));
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
 static void
 goa_owncloud_provider_init (GoaOwncloudProvider *self)
 {
@@ -1119,7 +1080,6 @@ goa_owncloud_provider_class_init (GoaOwncloudProviderClass *klass)
   provider_class->add_account                = add_account;
   provider_class->refresh_account            = refresh_account;
   provider_class->build_object               = build_object;
-  provider_class->show_account               = show_account;
   provider_class->ensure_credentials_sync    = ensure_credentials_sync;
 }
 
