@@ -34,6 +34,7 @@
 #include "goatelepathyfactory.h"
 #include "goapocketprovider.h"
 #include "goamediaserverprovider.h"
+#include "goalastfmprovider.h"
 
 #ifdef GOA_KERBEROS_ENABLED
 #include "goakerberosprovider.h"
@@ -906,6 +907,9 @@ static struct
 #ifdef GOA_EXCHANGE_ENABLED
   { GOA_EXCHANGE_NAME, goa_exchange_provider_get_type },
 #endif
+#ifdef GOA_LASTFM_ENABLED
+  { GOA_LASTFM_NAME, goa_lastfm_provider_get_type },
+#endif
 #ifdef GOA_IMAP_SMTP_ENABLED
   { GOA_IMAP_SMTP_NAME, goa_imap_smtp_provider_get_type },
 #endif
